@@ -2,7 +2,7 @@
 
 from math import inf
 
-import tinycss2.color4
+import tinycss2.color5
 
 from ..formatting_structure import boxes
 from ..logger import LOGGER
@@ -184,14 +184,16 @@ def table_layout(context, table, bottom_space, skip_stack, containing_block,
                     context, cell, bottom_space, cell_skip_stack,
                     page_is_empty=page_is_empty, absolute_boxes=absolute_boxes,
                     fixed_boxes=fixed_boxes, adjoining_margins=None,
-                    discard=False, max_lines=None)
+                    first_letter_style=None, first_line_style=None, discard=False,
+                    max_lines=None)
                 cell.style = original_style
                 if new_cell is None:
                     cell = cell.copy_with_children([])
                     cell, _, _, _, _, _ = block_container_layout(
                         context, cell, bottom_space, cell_skip_stack,
                         page_is_empty=True, absolute_boxes=[], fixed_boxes=[],
-                        adjoining_margins=None, discard=False, max_lines=None)
+                        adjoining_margins=None, first_letter_style=None,
+                        first_line_style=None, discard=False, max_lines=None)
                     cell_resume_at = {0: None}
                 else:
                     cell = new_cell
@@ -414,9 +416,13 @@ def table_layout(context, table, bottom_space, skip_stack, containing_block,
                     if avoid_page_break(page_break, context):
                         earlier_page_break = find_earlier_page_break(
                             context, new_table_children, absolute_boxes, fixed_boxes)
-                        if earlier_page_break is not None:
-                            new_table_children, resume_at = earlier_page_break
-                            break
+                        if earlier_page_break is None:
+                            remove_placeholders(
+                                context, new_table_children, absolute_boxes,
+                                fixed_boxes)
+                            return None, None, next_page, position_y
+                        new_table_children, resume_at = earlier_page_break
+                        break
                     resume_at = {index_group: None}
                 else:
                     return None, None, next_page, position_y
@@ -538,7 +544,8 @@ def table_layout(context, table, bottom_space, skip_stack, containing_block,
                 # We could not fit any content, drop the footer.
                 footer = None
 
-        assert not (header or footer)
+        assert not header
+        assert not footer
         new_table_children, resume_at, next_page, end_position_y = (
             body_groups_layout(skip_stack, position_y, bottom_space, page_is_empty))
         return header, new_table_children, footer, end_position_y, resume_at, next_page
@@ -566,6 +573,8 @@ def table_layout(context, table, bottom_space, skip_stack, containing_block,
         ([header] if header is not None else []) +
         new_table_children +
         ([footer] if footer is not None else []))
+    table.column_groups = tuple(
+        column_group.deepcopy() for column_group in table.column_groups)
     remove_end_decoration = resume_at is not None and not has_footer
     table.remove_decoration(remove_start_decoration, remove_end_decoration)
     if collapse:
@@ -780,7 +789,7 @@ def auto_table_layout(context, box, containing_block):
 
     if assignable_width < sum(max_content_guess):
         # Default values shouldn't be used, but we never know.
-        # See https://github.com/Kozea/WeasyPrint/issues/770
+        # See issue #770.
         lower_guess = guesses[0]
         upper_guess = guesses[-1]
 
@@ -931,7 +940,7 @@ def distribute_excess_width(context, grid, excess_width, column_widths, constrai
         column_widths[i] += excess_width / len(columns)
 
 
-TRANSPARENT = tinycss2.color4.parse_color('transparent')
+TRANSPARENT = tinycss2.color5.parse_color('transparent')
 
 
 def collapse_table_borders(table, grid_width, grid_height):
